package lsedit;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.text.NumberFormat;

import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JViewport;

public class ClusterMetrics extends JDialog implements ActionListener {

	protected Font				m_font;
	protected Font				m_bold;

	protected JPanel			m_left;
	protected JPanel			m_right;
	protected JButton			m_ok;

	protected EntityInstance	m_drawRoot;
	protected int				m_internal_edges;
	protected int				m_external_edges;
	protected int				m_boxes_seen;
	protected int				m_leaf_nodes;
	protected int				m_max_children;
	protected int				m_min_children;
	protected int				m_total_children;
	protected int				m_max_depth;
		
	public ClusterMetrics(LandscapeEditorCore ls, EntityInstance drawRoot) 	
	{
		super(ls.getFrame(), "Cluster Metrics", true);

		JFrame			frame;
		Container		contentPane;
		JPanel			panel;
		Font			font, bold;

		m_drawRoot    = drawRoot;
		m_font = font = FontCache.getDialogFont();
		m_bold = bold = font.deriveFont(Font.BOLD);
		
		frame = ls.getFrame();
		setLocation(frame.getX()+200, frame.getY()+300);
		setForeground(ColorCache.get(0,0,0));
		setBackground(ColorCache.get(192,192,192));
		setFont(font);

		contentPane = getContentPane();
		m_left      = new JPanel();
		m_left.setLayout(new GridLayout(0,1));
		m_right     = new JPanel();
		m_right.setLayout(new GridLayout(0,1));

		contentPane.add(m_left, BorderLayout.WEST);
		contentPane.add(m_right,BorderLayout.EAST);
		 
		panel = new JPanel();
		panel.setLayout(new FlowLayout());

		m_ok = new JButton("Ok");
		m_ok.setFont(bold);
		panel.add(m_ok);
		m_ok.addActionListener(this);

		contentPane.add(panel, BorderLayout.SOUTH);

		m_internal_edges = 0;
		m_external_edges = 0;
		m_boxes_seen     = 0;
		m_leaf_nodes     = 0;
		m_max_children   = 0;
		m_min_children   = 0;
		m_total_children = 0;
		m_max_depth      = 0;
	}

	public void seenRelation(RelationInstance ri)
	{
		RelationClass	rc;
		EntityInstance	src, dst;

		rc = ri.getRelationClass();
		if (rc.isContainsClass()) {
			return;
		}

		src = ri.getSrc();
		dst = ri.getDst();
		if (src.getContainedBy() == dst.getContainedBy()) {
			++m_internal_edges;
		} else {
			++m_external_edges;
	}	}

	public void seenEntity(EntityInstance e, int depth)
	{
		int	children = e.numChildren();

		if (depth > m_max_depth) {
			m_max_depth = depth;
		}
		if (children == 0) {
			++m_leaf_nodes;
		} else {
			++m_boxes_seen;
			m_total_children += children;

			if (m_min_children == 0) {
				m_min_children = children;
			} else if (m_min_children > children) {
				m_min_children = children;
			}
			if (m_max_children < children) {
				m_max_children = children;
		}	}
	}	

	public void showit()
	{
		int			 i;
		String		 s1, s2;
		JLabel		 label;
		NumberFormat format = NumberFormat.getInstance();
		format.setMaximumFractionDigits(2);


		for (i = 0; i < 9; ++i) {
			switch (i) {
			case 0:
				s1 = "Rooted at:";
				s2 = m_drawRoot.getFullEntityLabel();
				break;
			case 1:
				s1 = "Internal edges:";
				s2 = "" + m_internal_edges;
				break;
			case 2:
				s1 = "External edges:";
				s2 = "" + m_external_edges;
				break;
			case 3:
				s1 = "Minimum children:";
				s2 = "" + m_min_children;
				break;
			case 4:
				s1 = "Maximum children:";
				s2 = "" + m_max_children;
				break;
			case 5:
				if (m_boxes_seen == 0) {
					continue;
				}
				s1 = "Average children:";
				s2 = format.format((double) ((double) m_total_children) / ((double) m_boxes_seen));
				break;
			case 6:
				s1 = "Leaf nodes:";
				s2 = "" + m_leaf_nodes;
				break;
			case 7:
				s1 = "Total entities:";
				s2 = "" + (m_leaf_nodes + m_boxes_seen + 1);
				break;
			case 8:
				s1 = "Maximum depth:";
				s2 = "" + m_max_depth;
				break;
			default:
				s1 = "????";
				s2 = "????";
			}
			label = new JLabel(s1 + " ", JLabel.RIGHT);
			label.setFont(m_font);
			m_left.add(label);
			label = new JLabel(s2);
			label.setFont(m_bold);
			m_right.add(label);
		}

		// Resize the window to the preferred size of its components

		this.pack();
		setVisible(true);
	}

	// ActionListener interface

	public void actionPerformed(ActionEvent ev)
	{
		Object	source;

		// Pop down the window when the button is clicked.
		// System.out.println("event: " + ev);

		source = ev.getSource();

		if (source == m_ok) {
			this.setVisible(false);
			return;
		}
	}
}

